@@ -260,7 +260,7 @@ void Main_OnInitDialog (HWND hDlg)
     GetString (szUser, IDS_UNKNOWN);
 
     HKEY hk;
-    if (RegOpenKey (HKEY_LOCAL_MACHINE, REGSTR_PATH_AFS, &hk) == 0)
+    if (RegOpenKey (HKEY_LOCAL_MACHINE, AFSREG_CLT_SW_VERSION_SUBKEY, &hk) == 0)
     {
         DWORD dwSize = sizeof(szVersion);
         DWORD dwType = REG_SZ;
@@ -333,7 +333,8 @@ void Main_OnCheckMenuRemind (void)
 {
    BOOL fRemind = FALSE;
    lock_ObtainMutex(&g.credsLock);
-   for (size_t iCreds = 0; iCreds < g.cCreds; ++iCreds)
+   size_t iCreds;
+   for (iCreds = 0; iCreds < g.cCreds; ++iCreds)
       {
       if (g.aCreds[ iCreds ].fRemind)
          fRemind = TRUE;
@@ -414,14 +415,14 @@ void Main_OnCheckTerminate (void)
 {
     HKEY hk;
 
-    if (RegOpenKey (HKEY_CURRENT_USER, REGSTR_PATH_OPENAFS_CLIENT, &hk) == 0)
+    if (RegOpenKey (HKEY_CURRENT_USER, AFSREG_USER_OPENAFS_SUBKEY, &hk) == 0)
     {
         DWORD dwSize = sizeof(g.fStartup);
         DWORD dwType = REG_DWORD;
         RegQueryValueEx (hk, TEXT("ShowTrayIcon"), NULL, &dwType, (PBYTE)&g.fStartup, &dwSize);
         RegCloseKey (hk);
     }
-    else if (RegOpenKey (HKEY_LOCAL_MACHINE, REGSTR_PATH_OPENAFS_CLIENT, &hk) == 0)
+    else if (RegOpenKey (HKEY_LOCAL_MACHINE, AFSREG_CLT_OPENAFS_SUBKEY, &hk) == 0)
     {
       DWORD dwSize = sizeof(g.fStartup);
       DWORD dwType = REG_DWORD;
@@ -498,7 +499,8 @@ void Main_RepopulateTabs (BOOL fDestroyInvalid)
 
       size_t nCreds = 0;
       lock_ObtainMutex(&g.credsLock);
-      for (size_t iCreds = 0; iCreds < g.cCreds; ++iCreds)
+      size_t iCreds;
+      for (iCreds = 0; iCreds < g.cCreds; ++iCreds)
          {
          if (g.aCreds[ iCreds ].szCell[0])
             ++nCreds;
@@ -531,7 +533,8 @@ void Main_RepopulateTabs (BOOL fDestroyInvalid)
          {
          if (g.aCreds[ iCreds ].szCell[0])
             {
-            for (size_t ii = 0; ii < iTabOut; ++ii)
+            size_t ii;
+            for (ii = 0; ii < iTabOut; ++ii)
                {
                if (!ISCELLTAB (aTabs[ii]))
                   continue;
@@ -740,7 +743,7 @@ void Terminate_OnOK (HWND hDlg)
    g.fStartup = IsDlgButtonChecked (hDlg, IDC_STARTUP);
 
     HKEY hk;
-    if (RegCreateKey (HKEY_CURRENT_USER, REGSTR_PATH_OPENAFS_CLIENT, &hk) == 0)
+    if (RegCreateKey (HKEY_CURRENT_USER, AFSREG_USER_OPENAFS_SUBKEY, &hk) == 0)
     {
         DWORD dwSize = sizeof(g.fStartup);
         DWORD dwType = REG_DWORD;
